@@ -584,6 +584,16 @@ SKIP: {
 	}
 	pass('no crash with version->new($tied) where $tied returns "version"');
     }
+
+    { # [perl #112478]
+	$_112478::VERSION = 9e99;
+	ok eval { _112478->VERSION(9e99); 1 }, '->VERSION(9e99) succeeds'
+	    or diag $@;
+	$_112478::VERSION = 1;
+	eval { _112478->VERSION(9e99) };
+	like $@, qr/this is only/,
+	    '->VERSION(9e99) fails with the right error';
+    }
 }
 
 1;
